@@ -15,6 +15,15 @@
 
 #include "lightdm/power.h"
 
+/**
+ * SECTION:power
+ * @title: Power Management
+ * @short_description: Shutdown, restart, sleep the system
+ * @include: lightdm.h
+ *
+ * Helper functions to perform power management operations.
+ */
+
 static GDBusProxy *upower_proxy = NULL;
 static GDBusProxy *ck_proxy = NULL;
 static GDBusProxy *login1_proxy = NULL;
@@ -48,8 +57,6 @@ upower_call_function (const gchar *function, GError **error)
 static GVariant *
 login1_call_function (const gchar *function, GVariant *parameters, GError **error)
 {
-    GVariant *r;
-
     if (!login1_proxy)
     {
         login1_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
@@ -64,15 +71,39 @@ login1_call_function (const gchar *function, GVariant *parameters, GError **erro
             return NULL;
     }
 
-    r = g_dbus_proxy_call_sync (login1_proxy,
-                                function,
-                                parameters,
-                                G_DBUS_CALL_FLAGS_NONE,
-                                -1,
-                                NULL,
-                                error);
+    return g_dbus_proxy_call_sync (login1_proxy,
+                                   function,
+                                   parameters,
+                                   G_DBUS_CALL_FLAGS_NONE,
+                                   -1,
+                                   NULL,
+                                   error);
+}
 
-    return r;
+static GVariant *
+ck_call_function (const gchar *function, GVariant *parameters, GError **error)
+{
+    if (!ck_proxy)
+    {
+        ck_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                  G_DBUS_PROXY_FLAGS_NONE,
+                                                  NULL,
+                                                  "org.freedesktop.ConsoleKit",
+                                                  "/org/freedesktop/ConsoleKit/Manager",
+                                                  "org.freedesktop.ConsoleKit.Manager",
+                                                  NULL,
+                                                  error);
+        if (!ck_proxy)
+            return FALSE;
+    }
+
+    return g_dbus_proxy_call_sync (ck_proxy,
+                                   function,
+                                   parameters,
+                                   G_DBUS_CALL_FLAGS_NONE,
+                                   -1,
+                                   NULL,
+                                   error);
 }
 
 /**
@@ -85,27 +116,33 @@ login1_call_function (const gchar *function, GVariant *parameters, GError **erro
 gboolean
 lightdm_get_can_suspend (void)
 {
+    g_autoptr(GVariant) r = login1_call_function ("CanSuspend", NULL, NULL);
     gboolean can_suspend = FALSE;
-    GVariant *r;
-
-    r = login1_call_function ("CanSuspend", NULL, NULL);
     if (r)
     {
-        gchar *result;
+        const gchar *result;
         if (g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
         {
             g_variant_get (r, "(&s)", &result);
             can_suspend = g_strcmp0 (result, "yes") == 0;
         }
     }
-    else
+    if (!r)
+    {
+        const gchar *result;
+        r = ck_call_function ("CanSuspend", NULL, NULL);
+        if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
+        {
+            g_variant_get (r, "(&s)", &result);
+            can_suspend = g_strcmp0 (result, "yes") == 0;
+        }
+    }
+    if (!r)
     {
         r = upower_call_function ("SuspendAllowed", NULL);
         if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(b)")))
             g_variant_get (r, "(b)", &can_suspend);
     }
-    if (r)
-        g_variant_unref (r);
 
     return can_suspend;
 }
@@ -121,23 +158,22 @@ lightdm_get_can_suspend (void)
 gboolean
 lightdm_suspend (GError **error)
 {
-    GVariant *result;
-    gboolean suspended;
+    g_autoptr(GError) login1_error = NULL;
+    g_autoptr(GVariant) login1_result = login1_call_function ("Suspend", g_variant_new("(b)", FALSE), &login1_error);
+    if (login1_result)
+        return TRUE;
 
-    result = login1_call_function ("Suspend", g_variant_new("(b)", FALSE), error);
-    if (!result)
-    {
-        if (error)
-            g_debug ("Can't suspend using logind; falling back to UPower: %s", (*error)->message);
-        g_clear_error (error);
-        result = upower_call_function ("Suspend", error);
-    }
+    g_debug ("Can't suspend using logind; falling back to ConsoleKit: %s", login1_error->message);
 
-    suspended = result != NULL;
-    if (result)
-        g_variant_unref (result);
+    g_autoptr(GError) ck_error = NULL;
+    g_autoptr(GVariant) ck_result = ck_call_function ("Suspend", g_variant_new ("(b)", FALSE), &ck_error);
+    if (ck_result)
+        return TRUE;
 
-    return suspended;
+    g_debug ("Can't suspend using logind or ConsoleKit; falling back to UPower: %s", ck_error->message);
+
+    g_autoptr(GVariant) upower_result = upower_call_function ("Suspend", error);
+    return upower_result != NULL;
 }
 
 /**
@@ -150,27 +186,33 @@ lightdm_suspend (GError **error)
 gboolean
 lightdm_get_can_hibernate (void)
 {
+    g_autoptr(GVariant) r = login1_call_function ("CanHibernate", NULL, NULL);
     gboolean can_hibernate = FALSE;
-    GVariant *r;
-
-    r = login1_call_function ("CanHibernate", NULL, NULL);
     if (r)
     {
-        gchar *result;
+        const gchar *result;
         if (g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
         {
             g_variant_get (r, "(&s)", &result);
             can_hibernate = g_strcmp0 (result, "yes") == 0;
         }
     }
-    else
+    if (!r)
+    {
+        const gchar *result;
+        r = ck_call_function ("CanHibernate", NULL, NULL);
+        if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
+        {
+            g_variant_get (r, "(&s)", &result);
+            can_hibernate = g_strcmp0 (result, "yes") == 0;
+        }
+    }
+    if (!r)
     {
         r = upower_call_function ("HibernateAllowed", NULL);
         if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(b)")))
             g_variant_get (r, "(b)", &can_hibernate);
     }
-    if (r)
-        g_variant_unref (r);
 
     return can_hibernate;
 }
@@ -186,53 +228,22 @@ lightdm_get_can_hibernate (void)
 gboolean
 lightdm_hibernate (GError **error)
 {
-    GVariant *result;
-    gboolean hibernated;
+    g_autoptr(GError) login1_error = NULL;
+    g_autoptr(GVariant) login1_result = login1_call_function ("Hibernate", g_variant_new("(b)", FALSE), &login1_error);
+    if (login1_result)
+        return TRUE;
 
-    result = login1_call_function ("Hibernate", g_variant_new("(b)", FALSE), error);
-    if (!result)
-    {
-        if (error)
-            g_debug ("Can't hibernate using logind; falling back to UPower: %s", (*error)->message);
-        g_clear_error (error);
-        result = upower_call_function ("Hibernate", error);
-    }
+    g_debug ("Can't hibernate using logind; falling back to ConsoleKit: %s", login1_error->message);
 
-    hibernated = result != NULL;
-    if (result)
-        g_variant_unref (result);
+    g_autoptr(GError) ck_error = NULL;
+    g_autoptr(GVariant) ck_result = ck_call_function ("Hibernate", g_variant_new ("(b)", FALSE), &ck_error);
+    if (ck_result)
+        return TRUE;
 
-    return hibernated;
-}
+    g_debug ("Can't hibernate using logind or ConsoleKit; falling back to UPower: %s", ck_error->message);
 
-static GVariant *
-ck_call_function (const gchar *function, GError **error)
-{
-    GVariant *r;
-
-    if (!ck_proxy)
-    {
-        ck_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
-                                                  G_DBUS_PROXY_FLAGS_NONE,
-                                                  NULL,
-                                                  "org.freedesktop.ConsoleKit",
-                                                  "/org/freedesktop/ConsoleKit/Manager",
-                                                  "org.freedesktop.ConsoleKit.Manager",
-                                                  NULL,
-                                                  error);
-        if (!ck_proxy)
-            return FALSE;
-    }
-
-    r = g_dbus_proxy_call_sync (ck_proxy,
-                                function,
-                                NULL,
-                                G_DBUS_CALL_FLAGS_NONE,
-                                -1,
-                                NULL,
-                                error);
-
-    return r;
+    g_autoptr(GVariant) upower_result = upower_call_function ("Hibernate", error);
+    return upower_result != NULL;
 }
 
 /**
@@ -245,13 +256,11 @@ ck_call_function (const gchar *function, GError **error)
 gboolean
 lightdm_get_can_restart (void)
 {
+    g_autoptr(GVariant) r = login1_call_function ("CanReboot", NULL, NULL);
     gboolean can_restart = FALSE;
-    GVariant *r;
-
-    r = login1_call_function ("CanReboot", NULL, NULL);
     if (r)
     {
-        gchar *result;
+        const gchar *result;
         if (g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
         {
             g_variant_get (r, "(&s)", &result);
@@ -260,12 +269,10 @@ lightdm_get_can_restart (void)
     }
     else
     {
-        r = ck_call_function ("CanRestart", NULL);
+        r = ck_call_function ("CanRestart", NULL, NULL);
         if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(b)")))
             g_variant_get (r, "(b)", &can_restart);
     }
-    if (r)
-        g_variant_unref (r);
 
     return can_restart;
 }
@@ -281,20 +288,13 @@ lightdm_get_can_restart (void)
 gboolean
 lightdm_restart (GError **error)
 {
-    GVariant *r;
-    gboolean restarted;
+    g_autoptr(GError) login1_error = NULL;
+    g_autoptr(GVariant) login1_result = login1_call_function ("Reboot", g_variant_new("(b)", FALSE), &login1_error);
+    if (login1_result)
+        return TRUE;
 
-    r = login1_call_function ("Reboot", g_variant_new("(b)", FALSE), error);
-    if (!r)
-    {
-        g_clear_error (error);
-        r = ck_call_function ("Restart", error);
-    }
-    restarted = r != NULL;
-    if (r)
-        g_variant_unref (r);
-
-    return restarted;
+    g_autoptr(GVariant) ck_result = ck_call_function ("Restart", NULL, error);
+    return ck_result != NULL;
 }
 
 /**
@@ -307,13 +307,11 @@ lightdm_restart (GError **error)
 gboolean
 lightdm_get_can_shutdown (void)
 {
+    g_autoptr(GVariant) r = login1_call_function ("CanPowerOff", NULL, NULL);
     gboolean can_shutdown = FALSE;
-    GVariant *r;
-
-    r = login1_call_function ("CanPowerOff", NULL, NULL);
     if (r)
     {
-        gchar *result;
+        const gchar *result;
         if (g_variant_is_of_type (r, G_VARIANT_TYPE ("(s)")))
         {
             g_variant_get (r, "(&s)", &result);
@@ -322,12 +320,10 @@ lightdm_get_can_shutdown (void)
     }
     else
     {
-        r = ck_call_function ("CanStop", NULL);
+        r = ck_call_function ("CanStop", NULL, NULL);
         if (r && g_variant_is_of_type (r, G_VARIANT_TYPE ("(b)")))
             g_variant_get (r, "(b)", &can_shutdown);
     }
-    if (r)
-        g_variant_unref (r);
 
     return can_shutdown;
 }
@@ -337,24 +333,17 @@ lightdm_get_can_shutdown (void)
  * @error: return location for a #GError, or %NULL
  *
  * Triggers a system shutdown.
- * 
+ *
  * Return value: #TRUE if shutdown initiated.
  **/
 gboolean
 lightdm_shutdown (GError **error)
 {
-    GVariant *r;
-    gboolean shutdown;
+    g_autoptr(GError) login1_error = NULL;
+    g_autoptr(GVariant) login1_result = login1_call_function ("PowerOff", g_variant_new("(b)", FALSE), &login1_error);
+    if (login1_result)
+        return TRUE;
 
-    r = login1_call_function ("PowerOff", g_variant_new("(b)", FALSE), error);
-    if (!r)
-    {
-        g_clear_error (error);
-        r = ck_call_function ("Stop", error);
-    }
-    shutdown = r != NULL;
-    if (r)
-        g_variant_unref (r);
-
-    return shutdown;
+    g_autoptr(GVariant) ck_result = ck_call_function ("Stop", NULL, error);
+    return ck_result != NULL;
 }
