@@ -4,9 +4,8 @@
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version. See http://www.gnu.org/copyleft/lgpl.html the full text of the
- * license.
+ * Software Foundation; either version 2 or version 3 of the License.
+ * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
 #ifndef QLIGHTDM_USER_H
@@ -24,6 +23,9 @@ class UsersModelPrivate;
 class Q_DECL_EXPORT UsersModel : public QAbstractListModel
 {
     Q_OBJECT
+
+    Q_ENUMS(UserModelRoles)
+
 public:
     explicit UsersModel(QObject *parent = 0);
     ~UsersModel();
@@ -34,7 +36,10 @@ public:
                          BackgroundRole,
                          SessionRole,
                          HasMessagesRole,
-                         ImagePathRole
+                         ImagePathRole,
+                         BackgroundPathRole,
+                         UidRole,
+                         IsLockedRole
     };
 
     int rowCount(const QModelIndex &parent) const;
@@ -49,6 +54,6 @@ private:
 
 };
 
-};
+}
 
 #endif // QLIGHTDM_USER_H
