@@ -21,12 +21,9 @@ G_BEGIN_DECLS
 #define USER_TYPE (user_get_type())
 #define USER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), USER_TYPE, User));
 
-typedef struct UserPrivate UserPrivate;
-
 typedef struct
 {
-    GObject      parent_instance;
-    UserPrivate *priv;
+    GObject parent_instance;
 } User;
 
 typedef struct
@@ -34,6 +31,8 @@ typedef struct
     GObjectClass parent_class;
 } UserClass;
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (User, g_object_unref)
+
 User *accounts_get_user_by_name (const gchar *username);
 
 User *accounts_get_current_user (void);
